// acetone, 2025
// I hate copyright of any kind. This is a public domain.
// Original source: http://git.community.i2p/acetone/i2pdtunnelwizard

#include "notepad.h"

const std::u8string Notepad::ConfigComment::type(Lang lang, TunnelType type)
{
    switch (lang)
    {
    case Lang::ru: {
        static const std::u8string ru {u8"# Тип туннеля:\n"};
        return ru + tunnelTypeDescription(lang, type);
    }
    default: {
        static const std::u8string en {u8"# Tunnel type. You have selected\n"};
        return en + tunnelTypeDescription(lang, type);
    }
    }
}

const char8_t *Notepad::ConfigComment::clientAddress(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Локальный адрес, на котором будет доступен туннель\n"
                                    "# Например, 127.0.0.1\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# Local address where the tunnel will be accessed\n"
                                    "# For example, 127.0.0.1\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::clientPort(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Локальный порт, на котором будет доступен туннель\n"
                                    "# Например, 8080\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# Local port where the tunnel will be accessed\n"
                                    "# For example, 8080\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::clientDestination(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# I2P-адрес сервера (*.b32.i2p)\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# I2P server address (*.b32.i2p)\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::clientDestinationPort(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Порт сервера в сети I2P\n"
                                    "# Можно не указывать, если по адресу доступен только один сервис\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# Server port on the I2P network\n"
                                    "# Can be omitted if only one service is available at the address\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::clientKeepAlive(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Интервал между отправками пингов на сервер для поддержания соединения\n"
                                    "# Для отключения используется значение 0 (значение по умолчанию)\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# Interval between sending pings to the server to maintain connection\n"
                                    "# To disable, the value 0 is used (default)\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::serverAddress(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Адрес локального интерфейса, который будет использован\n"
                                    "# для доступа к сервису (например, 192.168.0.2 или 127.0.0.1)\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# The address of the local interface to be used\n"
                                    "# to access the service (for example, 192.168.0.2 or 127.0.0.1)\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::serverHost(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Адрес сервиса, который будет доступен в сети I2P\n"
                                    "# Например, 192.168.0.3 или 127.0.0.1\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# The address of the service that will be available on the I2P\n"
                                    "# For example, 192.168.0.3 or 127.0.0.1\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::serverPort(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Порт сервиса, по которому он доступен в обычной сети\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# The service port on which it is accessible on the regular network\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::serverInport(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Порт, по которому сервис будет доступен в сети I2P\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# The port on which the service will be available on the I2P\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::serverBlinded(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Параметры для зашифрованного лизсета (b33 aka bb32).\n"
                                    "# Используется для дополнительной приватности сервера, делая невозможным\n"
                                    "# пассивное обнаружение сервиса через мониторинг флудфилов\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# Parameters for encrypted leaseset (b33 aka bb32).\n"
                                    "# Used for additional server privacy, making it impossible\n"
                                    "# to passively detect the service through monitoring floodfilles\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::serverBlindedWithPsk(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Используется зашифрованный лизсет с авторизацией по паролю.\n"
                                    "# Формат строки для указания пароля для одного пользователя:\n"
                                    "# i2cp.leaseSetClient.psk.УНИКАЛЬНОЕ_ЧИСЛО = ИМЯ_ПОЛЬЗОВАТЕЛЯ_ДЛЯ_УДОБСТВА:ПАРОЛЬ\n"
                                    "# Важно: пароль указывается в специфичной для I2P кодировке Base64.\n"
                                    "# Пароль на стороне клиента такой же, как на сервере. Указывается одним параметром\n"
                                    "# i2cp.leaseSetPrivKey = ПАРОЛЬ (также в кодировке Base64).\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# An encrypted leaseset with password authorization is used.\n"
                                 "# String format for specifying a password for one user: \n"
                                 "# i2cp.leaseSetClient.psk.UNICAL_NUMBER = OPTIONAL_USER_NAME:PASSWORD\n"
                                 "# Important: the password is specified in I2P-specific Base64 encoding.\n"
                                 "# The password on the client side is the same as on the server.\n"
                                 "# It is specified by one parameter \"i2cp.leaseSetPrivKey = PASSWORD\" (Base64 encoded).\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::serverBlindedWithDh(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Используется зашифрованный лизсет с авторизацией по ключу x25519.\n"
                                    "# Формат строки для указания ключа для одного пользователя:\n"
                                    "# i2cp.leaseSetClient.dh.УНИКАЛЬНОЕ_ЧИСЛО = ИМЯ_ПОЛЬЗОВАТЕЛЯ_ДЛЯ_УДОБСТВА:ПУБЛИЧНЫЙ_КЛЮЧ\n"
                                    "# Важно: публичный ключ указывается в специфичной для I2P кодировке Base64.\n"
                                    "# На стороне клиента указывается секретный ключ (один параметр):\n"
                                    "# i2cp.leaseSetPrivKey = СЕКРЕТНЫЙ_КЛЮЧ (также в кодировке Base64).\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# An encrypted leaseset with authorization by key x25519 is used.\n"
                                    "# The format of the string to specify the key for a single user is:\n"
                                    "# i2cp.leaseSetClient.dh.UNICAL_NUMBER = OPTIONAL_USER_NAME:PUBLIC_KEY\n"
                                    "# Important: the public key is specified in I2P-specific Base64 encoding.\n"
                                    "# The secret key is specified on the client side. It is specified by one parameter:\n"
                                    "# i2cp.leaseSetPrivKey = SECRET_KEY (also in Base64 encoding).\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::serverBlindedWithDhPrivateKey(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Секретный ключ для "};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# A secret key for the "};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::serverBlindedWithPassword(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Пароль для "};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# A password for "};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::inbound(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Это конфиг туннеля, который является логической сущностью, обеспечивающей\n"
                                    "# соединение через сеть I2P. Фактически туннель состоит из нескольких туннелей,\n"
                                    "# которые называются нитями.\n"
                                    "# Параметры входящих нитей:\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# This is a tunnel config, which is a logical term that enables communication.\n"
                                    "# In fact, a tunnel consists of several tunnels, which are called strands.\n"
                                    "# Parameters of inbound strands:\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::outbound(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Параметры исходящих нитей:\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# Parameters of outbound strands:\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::length(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# - Количество транзитных узлов (длина)\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# - Number of transit nodes (length)\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::quantity(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# - Количество нитей\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# - Strands count\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::variance(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# - Число, на которое будет меняться длина случайных нитей (от -3 до 3)\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# - The number by which the length of random strands will be changed (from -3 to 3)\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::keysTransient(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Временные ключи. Адрес туннеля будет меняться после каждого запуска i2pd.\n"
                                    "# Чтобы использовать одинаковый временный адрес на нескольких туннелях,\n"
                                    "# указывайте одинаковое название transient-*\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# Temporary keys. The tunnel address will change after each i2pd startup.\n"
                                    "# To use the same temporary address on multiple tunnels,\n"
                                    "# specify the same transient-* name\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::keys(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Ключи туннеля, обеспечивающие постоянный адрес. Если указанный файл не существует,\n"
                                    "# ключи будут созданы при первом запуске. Ключи хранятся в рабочей директории i2pd\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# Tunnel keys that provide a permanent address. If the specified file does not exist,\n"
                                    "# keys will be created at the first startup. Keys are stored in the i2pd working directory\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::ssl(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Если этот параметр равен true, i2pd будет подключаться к сервису, который выводится\n"
                                    "# в сеть I2P, используя защищенное соединение. Например, к HTTPS-сайту\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# If this parameter is true, i2pd will connect to the service that is brought onto the I2P\n"
                                    "# network by this tunnel using a secure connection. For example, to an HTTPS site\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::hostoverride(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Значение для подстановки в HTTP-заголовок \"Host\"\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# Value to be substituted into HTTP header \"Host\"\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::maxStreamCount(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Максимальное количество входящих TCP-подключений в один момент времени. По умолчанию 2048.\n"
                                    "# Ограничение позволяет бороться с DDoS-атакой, в которой атакующий парализует работу i2pd\n"
                                    "# через достижение лимита на количество файловых дескрипторов. Этот параметр ограничивает\n"
                                    "# количество файловых дескрипторов, которое доступно туннелю. В случае атаки, пострадает\n"
                                    "# только один туннель - тот, который подвергся атаке. Другие туннели продолжат штатную работу.\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# Maximum number of incoming TCP connections at one point in time. The default is 2048.\n"
                                    "# The restriction allows you to fight a DDoS attack in which an attacker paralyzes i2pd\n"
                                    "# through reaching the limit on the number of file descriptors. This parameter limits the number\n"
                                    "# of file descriptors that are available to the tunnel. In case of an attack, only one tunnel\n"
                                    "# will be affected - the one that was attacked. Other tunnels will continue normal operation.\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::httpOutproxy(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Адрес выходного HTTP-прокси (\"аутпрокси\"). Будет использован при запросах на адреса вне сети I2P.\n"
                                    "# Аутпрокси должен иметь префикс \"http://\" и адрес *.i2p. Также поддерживается указание порта.\n"
                                    "# Пример: http://outproxy.acetone.i2p:3128\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# HTTP output proxy address (\"outproxy\"). It will be used when requesting addresses outside the\n"
                                    "# I2P network. The outproxy must have the prefix \"http://\" and an *.i2p address. Port specification\n"
                                    "# is also supported. Example: http://outproxy.acetone.i2p:3128\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::socksOutproxy(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Адрес выходного SOCKS4-прокси (\"аутпрокси\"). Будет использован при запросах на адреса\n"
                                    "# вне сети I2P. Работа с сервером в I2P не поддерживается. Для использования выходного сервера\n"
                                    "# в I2P создайте клиентские туннели TCP и UDP на одном порту и укажите адрес здесь.\n"
                                    "# Важно: поддерживается только SOCKS4. Пример: 127.0.0.1 и порт 9050\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# The address of the output SOCKS4 proxy (\"outproxy\"). It will be used when requesting\n"
                                    "# addresses outside the I2P network. Server operation in I2P is not supported. To use the output\n"
                                    "# server in I2P, create TCP and UDP client tunnels on the same port and specify the address here.\n"
                                    "# Important: only SOCKS4 is supported. Example: 127.0.0.1 and port 9050\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::ircPassword(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Пароль для отправки с командой WEBIRC\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# Password to send with WEBIRC command\n"};
        return en;
    }
    }
}

const char8_t *Notepad::ConfigComment::b33ClientKey(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"# Укажите пароль или секретный ключ для подключения к серверу, который использует\n"
                                    "# зашифрованный лизсет (используется специфичная для I2P кодировка Base64)\n"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"# Specify a password or secret key to connect to a server that uses\n"
                                    "# encrypted leaseset (I2P-specific Base64 encoding is used)\n"};
        return en;
    }
    }
}

const std::u8string Notepad::ConfigComment::footer(Lang lang, bool isBlinded, const std::string& tunnelname)
{
    std::u8string str;
    switch (lang)
    {
    case Lang::ru: {
        if (isBlinded)
        {
            static const char8_t* ru0 {u8"\n"
                                       "# Для создания туннеля нужно вставить этот текст в начало конфига tunnels.conf,\n"
                                       "# который находится в рабочей директории i2pd. Расположение рабочей директории\n"
                                       "# в системе отображается на главной странице веб-консоли i2pd (http://127.0.0.1:7070).\n"
                                       "# Адрес созданного туннеля можно найти также через веб-консоль\n"
                                       "# по пути I2P Tunnels -> "};
            static const char8_t* ru1 {u8" -> Encrypted B33 address.\n"
                                       "# Другие параметры и более подробное описание приведенных параметров смотрите\n"
                                       "# в документации: https://i2pd.readthedocs.io/en/latest/user-guide/tunnels/"};
            str = ru0;
            str += reinterpret_cast<const char8_t*>(tunnelname.c_str());
            str += ru1;
            return str;
        }
        else
        {
            static const char8_t* ru {u8"\n"
                                       "# Для создания туннеля нужно вставить этот текст в начало конфига tunnels.conf,\n"
                                       "# который находится в рабочей директории i2pd. Расположение рабочей директории\n"
                                       "# в системе отображается на главной странице веб-консоли i2pd (http://127.0.0.1:7070).\n"
                                       "# Адрес созданного туннеля можно найти также через веб-консоль, вкладка I2P Tunnels.\n"
                                       "# Другие параметры и более подробное описание приведенных параметров смотрите\n"
                                       "# в документации: https://i2pd.readthedocs.io/en/latest/user-guide/tunnels/"};
            return ru;
        }
    }
    default: {
        if (isBlinded)
        {
            static const char8_t* en0 {u8"\n"
                                     "# To create a tunnel, you need to paste this text into the beginning of the tunnels.conf config,\n"
                                     "# which is located in the i2pd working directory. Location of the working directory\n"
                                     "# in the system is displayed on the main page of the i2pd web console (http://127.0.0.1:7070).\n"
                                     "# The address of the created tunnel can also be found through the web console\n"
                                     "# on the path I2P Tunnels -> "};
            static const char8_t* en1 {u8" -> Encrypted B33 address.\n"
                                     "# For other parameters and more detailed description of the above parameters see\n"
                                     "# in the documentation: https://i2pd.readthedocs.io/en/latest/user-guide/tunnels/"};
            str = en0;
            str += reinterpret_cast<const char8_t*>(tunnelname.c_str());
            str += en1;
            return str;
        }
        else
        {
            static const char8_t* en {u8"\n"
                                     "# To create a tunnel, you need to paste this text into the beginning of the tunnels.conf config,\n"
                                     "# which is located in the i2pd working directory. Location of the working directory\n"
                                     "# in the system is displayed on the main page of the i2pd web console (http://127.0.0.1:7070).\n"
                                     "# The address of the created tunnel can also be found through the web console, I2P Tunnels tab.\n"
                                     "# For other parameters and more detailed description of the above parameters see\n"
                                     "# in the documentation: https://i2pd.readthedocs.io/en/latest/user-guide/tunnels/"};
            return en;
        }
    }
    }
}

std::string Notepad::langToString(Lang lang)
{
    return lang == Lang::ru ? "ru" : "en";
}

std::string Notepad::langToCode(Lang lang)
{
    switch (lang)
    {
    case Lang::ru:
        return "ru";
    default:
        return "en";
    }
}

Notepad::Lang Notepad::stringToLang(const std::string &string)
{
    return string == "ru" ? Lang::ru : Lang::en;
}

const char8_t *Notepad::tunnelTypeDescription(Lang lang, TunnelType type)
{
    switch (type)
    {
    case TunnelType::TcpClient: {
        switch (lang)
        {
        case Lang::ru: {
            static const char8_t* ru {u8"# Клиентское подключение по протоколу TCP\n"};
            return ru;
        }
        default: {
            static const char8_t* en {u8"# Client connection via TCP protocol\n"};
            return en;
        }
        }
    }
    case TunnelType::TcpServer: {
        switch (lang)
        {
        case Lang::ru: {
            static const char8_t* ru {u8"# Серверный туннель для приема подключений по протоколу TCP\n"};
            return ru;
        }
        default: {
            static const char8_t* en {u8"# Server tunnel to accept TCP connections\n"};
            return en;
        }
        }
    }
    case TunnelType::UdpClient: {
        switch (lang)
        {
        case Lang::ru: {
            static const char8_t* ru {u8"# Клиентское подключение по протоколу UDP\n"};
            return ru;
        }
        default: {
            static const char8_t* en {u8"# Client connection via UDP protocol\n"};
            return en;
        }
        }
    }
    case TunnelType::UdpServer: {
        switch (lang)
        {
        case Lang::ru: {
            static const char8_t* ru {u8"# Серверный туннель для приема подключений по протоколу UDP\n"};
            return ru;
        }
        default: {
            static const char8_t* en {u8"# Server tunnel to accept UDP connections\n"};
            return en;
        }
        }
    }
    case TunnelType::HttpServer: {
        switch (lang)
        {
        case Lang::ru: {
            static const char8_t* ru {u8"# Серверный туннель для приема подключений по протоколу TCP\n"
                                        "# с добавлением специальных заголовков HTTP к каждому запросу\n"
                                        "# для идентификации клиента\n"};
            return ru;
        }
        default: {
            static const char8_t* en {u8"# Server tunnel to accept TCP connections with adding\n"
                                        "# special HTTP headers to each request to identify the client\n"};
            return en;
        }
        }
    }
    case TunnelType::HttpProxy: {
        switch (lang)
        {
        case Lang::ru: {
            static const char8_t* ru {u8"# HTTP-прокси для выхода в сеть I2P\n"};
            return ru;
        }
        default: {
            static const char8_t* en {u8"# HTTP proxy for I2P network access\n"};
            return en;
        }
        }
    }
    case TunnelType::SocksProxy: {
        switch (lang)
        {
        case Lang::ru: {
            static const char8_t* ru {u8"# SOCKS5-прокси для выхода в сеть I2P\n"};
            return ru;
        }
        default: {
            static const char8_t* en {u8"# SOCKS5 proxy for I2P network access\n"};
            return en;
        }
        }
    }
    case TunnelType::IrcServer: {
        switch (lang)
        {
        case Lang::ru: {
            static const char8_t* ru {u8"# Серверный туннель для приема подключений по протоколу TCP\n"
                                        "# с добавлением специального заголовка для отображения I2P-адресов\n"
                                        "# клиентов, которые подключаются к серверу IRC (Internet Relay Chat)\n"};
            return ru;
        }
        default: {
            static const char8_t* en {u8"# Server tunnel to accept TCP connections\n"
                                        "# with addition of a special header for displaying I2P addresses\n"
                                        "# of clients that connect to the IRC (Internet Relay Chat) server\n"};
            return en;
        }
        }
    }
    default: {
        switch (lang)
        {
        case Lang::ru: {
            static const char8_t* ru {u8"# ^-^ это баг\n"};
            return ru;
        }
        default: {
            static const char8_t* en {u8"# ^-^ this is a bug\n"};
            return en;
        }
        }
    }
    }
}

const char8_t *Notepad::SetterError::name(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Название не может содержать квадратные скобки, т.к. квадратные скобки являются служебным символом формата ini, "
                                    "который используется в конфиге i2pd (также название не должно быть пустым)"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"The name cannot contain square brackets, because square brackets are a service character of the ini format, "
                                    "which is used in the i2pd config (also the name should not be empty)"};
        return en;
    }
    }
}

const char8_t *Notepad::SetterError::length(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Максимальная длина туннеля составляет 8 транзитных узлов (минимальная - 0)"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"The maximum length of the tunnel is 8 transit nodes (minimum is 0)"};
        return en;
    }
    }
}

const char8_t *Notepad::SetterError::variance(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Фактор случайного изменения длины нитей может принимать значения от -3 до 3."};
        return ru;
    }
    default: {
        static const char8_t* en {u8"The factor of random variation in strands length can take values from -3 to 3."};
        return en;
    }
    }
}

const char8_t *Notepad::SetterError::quantity(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Максимальная ширина туннеля (количество нитей) - 16. Минимальное - 1."};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Maximum tunnel width (number of strands) - 16. Minimum - 1."};
        return en;
    }
    }
}

const char8_t *Notepad::SetterError::maxStreamCount(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Максимальное количество TCP-стримов не может быть меньше 1."};
        return ru;
    }
    default: {
        static const char8_t* en {u8"The maximum number of TCP-streams cannot be less than 1."};
        return en;
    }
    }
}

const char8_t *Notepad::SetterError::b33UserCount(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Практически i2pd поддерживает до 64 000 пользователей зашифрованного лизсета, но этот сервис генерирует не более 30."};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Practically i2pd supports up to 64,000 encrypted leaseset users, but this service generates no more than 30."};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::errorNoRequiredParams(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Не переданы необходимые параметры"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Necessary parameters have not been passed"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::toStartTitle(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Вернуться на главную страницу"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Back to main page"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::startSelectTunnelType(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Выберите тип туннеля"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Select a tunnel type"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::startConfirm(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Выбрать"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Select"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::startAbout(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Этот сервис поможет вам сконфигурировать клиентские и серверные туннели i2pd. "
                                    "Основные опции протокола I2P доступны здесь без необходимости глубокого понимания архитектуры сети. "
                                    "Просто выберите тип туннеля, который вам нужен, затем произведите тонкую настройку через "
                                    "интуитивно понятные средства ввода и получите конфиг, готовый к использованию."};
        return ru;
    }
    default: {
        static const char8_t* en {u8"This service helps you configure i2pd client and server tunnels. "
                                    "General I2P protocol options are available here without the need for deep understanding of the network architecture. "
                                    "Simply select the type of tunnel you need, then fine tune it through intuitive input tools "
                                    "and you will have a config ready to use."};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::tunnelType(Lang lang, TunnelType type)
{
    switch (type)
    {
    case TunnelType::TcpServer: {
        return startDropdownServerTCP(lang);
    }
    case TunnelType::UdpClient: {
        return startDropdownClientUDP(lang);
    }
    case TunnelType::UdpServer: {
        return startDropdownServerUDP(lang);
    }
    case TunnelType::HttpServer: {
        return startDropdownServerHTTP(lang);
    }
    case TunnelType::IrcServer: {
        return startDropdownServerIRC(lang);
    }
    case TunnelType::SocksProxy: {
        return startDropdownSOCKSProxy(lang);
    }
    case TunnelType::HttpProxy: {
        return startDropdownHTTPProxy(lang);
    }
    default: {
        return startDropdownClientTCP(lang);
    }
    }
}

const char8_t *Notepad::WebUi::tagline(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Конструктор конфигов туннелей I2P"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"I2P tunnel config builder"};
        return en;
    }
    }
}

const char8_t * Notepad::WebUi::error(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Ошибка"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Error"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainThOption(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Параметр"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Parameter"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainThInput(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Значение"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Value"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainTunnelName(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Название туннеля"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Tunnel name"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainTunnelNamePlaceholder(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Латиница, цифры и _ - ."};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Letters, numbers, and _ - ."};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainTunnelType(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Тип туннеля"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Tunnel type"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::startDropdownClientTCP(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Клиент TCP"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"TCP client"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::startDropdownClientUDP(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Клиент UDP"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"UDP client"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::startDropdownServerTCP(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Сервер TCP"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"TCP server"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::startDropdownServerUDP(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Сервер UDP"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"UDP server"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::startDropdownServerHTTP(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Сервер HTTP"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"HTTP server"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::startDropdownServerIRC(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Сервер IRC"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"IRC server"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::startDropdownSOCKSProxy(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"SOCKS5-прокси"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"SOCKS5 proxy"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::startDropdownHTTPProxy(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"HTTP-прокси"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"HTTP proxy"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainInbound(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Входящие"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Inbound"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainOutbound(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Исходящие"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Outbound"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainLength(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Длина нитей (количество транзитных узлов)"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Tunnel length (number of transit nodes)"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainQuantity(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Количество нитей (ширина туннеля)"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Number of strands (tunnel width)"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainVariance(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Число для изменения длины случайных нитей"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Number to change the length of random strands"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainTransient(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Временные ключи (внутрисетевой адрес изменится после рестарта i2pd)"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Temporary keys (on-net address will change after i2pd restart)"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainKeepalive(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Пинг сервера для поддержания соединения (интервал в секундах; чтобы выключить - 0). Только для клиентских туннелей"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Server ping to keep the connection alive (interval in seconds; to turn off - 0). For client tunnels only"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainSsl(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"SSL-соединение с приложением, которое выводится в сеть I2P"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"SSL connection to an application that is brought up on an I2P network"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainB33KeysServer(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Количество клиентов туннеля для генерации ключей, которые будут включены в конфигурационный файл"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Number of tunnel clients for key generation to be included in the configuration file"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainB33PasswordServer(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Количество клиентов туннеля для генерации паролей, которые будут включены в конфигурационный файл"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Number of tunnel clients for password generation to be included in the configuration file"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainHostoverride(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Значение HTTP-заголовока \"Host\" надо заменять (по умолчанию передается полученный от клиента)"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"The value of HTTP headword \"Host\" should be replaced (by default the one received from the client is transmitted)"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainWebIrcPasswordServer(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Нужно использовать пароль при отправке команд WEBIRC"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Need to use password when sending WEBIRC commands"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainHttpOutproxyClient(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Будет использоваться выходной HTTP-прокси для визита на не-I2P ресурсы"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"An output HTTP proxy for visiting non-I2P resources will be used"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainSocksOutproxyClient(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Будет использоваться выходной SOCKS4-прокси для визита на не-I2P ресурсы"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"A SOCKS4 output proxy for visiting non-I2P resources will be used"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainClientB33(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Нужно добавить секрет (пароль или ключ) для подключения к серверу, который использует зашифрованный лизсет с авторизацией"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Need to add a secret (password or key) to connect to a server that uses an encrypted leaseset with authorization"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainComments(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Добавить подробные комментарии в конфиг"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Add detailed comments to the config"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::mainGenerate(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Сгенерировать"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Generate"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::configToStart(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"На главную"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Main page"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::b33TaglineServer(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Дополнительная приватность сервера"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Server extra privacy"};
        return en;
    }
    }
}

const std::u8string Notepad::WebUi::b33AboutServer(Lang lang, TunnelType type)
{
    std::u8string str;
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru0 {u8"I2P имеет несколько вариантов обеспечения дополнительной защиты от случайных посетителей для вашего сервиса типа \""};
        static const char8_t* ru1 {u8"\". При использовании зашифрованного лизсета вы получаете защиту от нежелательных запросов на уровне сети (у несанкционированного пользователя "
                                     "не будет возможности установить контакт с роутером сервера)."};
        str = ru0;
        str += tunnelType(lang, type);
        str += ru1;
        return str;
    }
    default: {
        static const char8_t* en0 {u8"I2P has several options for providing additional protection against accidental visitors for your service with type \""};
        static const char8_t* en1 {u8"\". When you use an encrypted leaseset, you get protection against unwanted requests at the network level (an unauthorized user will have no way "
                                     "to make contact with server router)."};
        str = en0;
        str += tunnelType(lang, type);
        str += en1;
        return str;
    }
    }
    return str;
}

const char8_t *Notepad::WebUi::b33ThOption(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Описание"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Description"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::b33ThInput(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Выбор"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Selection"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::b33NoneServer(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Без зашифрованного лизсета. Подходящий вариант для публичного сервиса. К ключам такого типа можно привязать короткий адрес *.i2p"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Without encrypted leaseset. Suitable option for a public service. A *.i2p short address can be bound to keys of this type"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::b33NoneShort(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Не нужно"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Don't need"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::b33SimpleServer(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Зашифрованный лизсет без авторизации. Препятствует пассивному обнаружению вашего сервиса через мониторинг флудфилов. "
                                    "Хороший вариант для приватного сервиса без дополнительной настройки под каждого пользователя, достаточно сообщить только адрес. "
                                    "К такому адресу нельзя привязать короткий домен *.i2p"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Encrypted leaseset without authorization. Prevents passive discovery of your service through monitoring of floodfilles. "
                                    "A good option for private service without additional customization for each user, you only need to provide the address. "
                                    "You cannot bind a *.i2p short domain to such an address"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::b33SimpleShort(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Без авторизации"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"No authorization"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::b33PasswordServer(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Зашифрованный лизсет с авторизацией по паролю. Для доступа к серверу недостаточно знать его адрес - нужен пароль. "
                                    "Рекомендуется задавать индивидуальный пароль для каждого пользователя (лимит пользователей - 64k). "
                                    "Для подключения, пользователь должен указать пароль в конфиге туннеля (параметр i2cp.leaseSetPrivKey), либо в конфигурации HTTP-прокси в i2pd.conf. "
                                    "К такому адресу нельзя привязать короткий домен *.i2p"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Encrypted leaseset with password authorization. It is not enough for a user to know address to access the server - he needs a password. "
                                    "It is recommended to set an individual password for each user (user limit - 64k). "
                                    "To connect, the user must specify a password in the tunnel config (i2cp.leaseSetPrivKey parameter), or in the HTTP proxy configuration in i2pd.conf. "
                                    "A *.i2p short domain cannot be bound to such an address"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::b33PasswordShort(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Пароль"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Password"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::b33KeysServer(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Зашифрованный лизсет с авторизацией по ключу x25519. Высшая степень защиты. Для доступа к серверу пользователю недостаточно знать адрес - нужен секретный ключ. "
                                    "Рекомендуется использовать индивидуальный ключ для каждого пользователя (лимит пользователей - 64k). "
                                    "Для подключения, пользователь должен указать свой секретный ключ в конфиге туннеля (параметр i2cp.leaseSetPrivKey), либо в конфигурации HTTP-прокси в i2pd.conf. "
                                    "Для шифрования лизсета сервер использует публичные ключи пользователей. К такому адресу нельзя привязать короткий домен *.i2p"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Encrypted leaseset with authorization by key x25519. Highest security level. To access the server, it is not enough for the user to know the address - a secret key is required. "
                                    "It is recommended to use an individual key for each user (user limit - 64k). To connect, the user must specify his secret key in the tunnel configuration "
                                    "(i2cp.leaseSetPrivKey parameter) or in the HTTP proxy configuration in i2pd.conf. The server uses users' public keys to encrypt the leaseset. "
                                    "You cannot bind a *.i2p short domain to such an address"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::b33KeysShort(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Ключи"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Keys"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::b33Confirm(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Далее"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Next"};
        return en;
    }
    }
}

const char8_t *Notepad::WebUi::goBack(Lang lang)
{
    switch (lang)
    {
    case Lang::ru: {
        static const char8_t* ru {u8"Назад"};
        return ru;
    }
    default: {
        static const char8_t* en {u8"Back"};
        return en;
    }
    }
}
